import os

import warnings

import numpy as np
import pandas as pd



class Diarization:
    def __init__(self, segments, frame_size = 10):
        self.segs = segments
        self.frame_size = frame_size

        # Calculate some useful things
        self.num_segments = self.segs.shape[0]

        last = self.segs.iloc[self.num_segments - 1]
        self.estimated_total_frames = last.start + last.length

        self.speakers = np.unique(self.segs.speaker)
        self.num_speakers = self.speakers.shape[0]


    def iter_segments(self):
        return self.segs.iterrows()

    def segment_at(self, i):
        return self.segs.iloc[i]


    def change_speaker(self, speaker, new_name):
        new_segs = self.segs.copy()
        new_segs.loc[self.segs.speaker == speaker, "speaker"] = new_name
        return self.update_segs(new_segs)


    def merge_equal_neighbors(self):
        """ Merge neighbors with same speaker. """
        IDX_LENGTH = 3

        merged = self.segs.copy()
        current_start = 0
        j = 0
        seg = self.segs.iloc[0]
        for i in range(1, self.num_segments):
            seg = self.segs.iloc[i]
            last = self.segs.iloc[i - 1]
            if seg.speaker == last.speaker:
                merged.iat[j, IDX_LENGTH] = seg.start + seg.length - current_start
            else:
                j += 1
                merged.iloc[j] = seg
                current_start = seg.start
        merged = merged.iloc[:(j+1)]
        merged.sort_values('start', inplace = True)

        return self.update_segs(merged)

    def update_segs(self, new_segs):
        return Diarization(segments = new_segs,
                           frame_size = self.frame_size)


    def frame_to_time(self, frame):
        return frame * self.frame_size



    def save(self, filename):
        self.segs.to_pickle(filename)

    @classmethod
    def load(cls, filename):
        segs = pd.read_pickle(filename)
        return Diarization(segments = segs)



import click

from lium_diarization_editor import Diarization
from lium_diarization_editor.lium import load_diarization as load_lium_diarization
from lium_diarization_editor.editor import DiarizationEditor
from lium_diarization_editor.editor import AudioPlayer


@click.group()
def cli():
    pass



@click.command(help='View or edit a diarization.')
@click.option(
    '--audio', '-a', 'audio_file',
    help='Load an audio file belonging to the diarization.')
@click.option(
    '--dia', '-d', 'dia_file',
    help='A LIUM diarization file.')
@click.option(
    '--pickled-dia', '-p', 'pickled_dia_file',
    help='A pickled diarization, as created using the --save option.')
@click.option(
    '--save', '-s', 'save_path',
    help='Where to save the diarization (as pickle) when pressing "s".')

def main(dia_file=None, pickled_dia_file=None, audio_file=None, save_path=None):
    if audio_file:
        player = AudioPlayer(audio_file)
    else:
        player = None

    if dia_file:
        diarization = load_lium_diarization(dia_file)
    else:
        if pickled_dia_file is None:
            print("One of --dia and --pickled-dia is required. "
                  "Run with --help to see all options.")
            exit(1)
        diarization = Diarization.load(pickled_dia_file)

    editor = DiarizationEditor(diarization, player, save_latest_path=save_path)
    editor.run()


if __name__ == '__main__':
    main()



import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command


NAME = 'lium_diarization_editor'
DESCRIPTION = 'Editor/viewer for LIUM diarizations.'
URL = 'https://github.com/maxhollmann/lium-diarization-editor'
EMAIL = 'maxhollmann@gmail.com'
AUTHOR = 'Max Hollmann'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = '0.1.1'

REQUIRED = [
    'python-mpv',
    'matplotlib',
    'click',
    'pandas',
]

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(VERSION))
        os.system('git push --tags')

        sys.exit()


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    python_requires=REQUIRES_PYTHON,
    packages=['lium_diarization_editor'],
    license='MIT',
    entry_points={
        'console_scripts': ['lium-dia-edit=lium_diarization_editor.cli:main'],
    },
    install_requires=REQUIRED,
    cmdclass={
        'upload': UploadCommand,
    },
)



import pandas as pd
from lium_diarization_editor import Diarization


def load_diarization(filename):
    with open(filename) as fp:
        lines = list(fp)
    lines = [l.strip().split(" ")
             for l in lines
             if not l.startswith(";;")]
    segs = pd.DataFrame(lines, columns=("show", "channel", "start", "length", "gender", "band", "environment", "speaker"))
    segs.start = pd.to_numeric(segs.start)
    segs.length = pd.to_numeric(segs.length)
    segs.sort_values('start', inplace=True)

    if 'original_speaker' not in segs:
        segs['original_speaker'] = segs['speaker']

    return Diarization(segments=segs)
